<?php
/*
 * Created on 30 nov. 2004
 */
require_once '../compat.php';
require_once  '../lib/Wiki_DB.php';


// Global Vars
$fs_separator = (substr(PHP_OS, 0, 3) == 'WIN') ? "\\" : "/" ;
@define('OS_FILE_SEPARATOR',$fs_separator);
umask(0000);
$umask = 0775;

/**
 * Extracts properties of a template file
 *
 */
function setup_parseTemplate($tpl_file, $raw = false) {
    $config = array();
    
    $t = file($tpl_file, 1);
    for ($x=0; $x<count($t); $x++) {
        $l = $t[$x];
        $l = trim($l);

        if ($l != '') {
            switch ($l[0]) {

                // New Configure section
                case '/':
                    if ($l[1] == '/') {
                        $current = trim(substr($l, 2));
                        $config['categories'][$current] = array();
                    }
                    break;

                case '#':
                    $value =  trim(substr($l, 1));
                    $config['descriptions'][$current] = (defined($value) ? constant($value) : $value); // find translation if constant used
                    break;

                // A configure option
                case '$':
                    // Grep out the name, type and default
                    preg_match('#\{([^|]+\|[^|]+\|([^}]+)?)\}[^/]+/{2}(.+)#msi', $l, $match);
                    $c    = explode('|', $match[1]);
                    $cdef = setup_queryDefault($c[1], $c[3]);
                    $config['categories'][$current][] = $configRaw[] = array(
                                            'longname'    => (defined($c[0]) ? constant($c[0]) : $c[0]), // find translation if constant used
                                            'name'        => $c[1],
                                            'type'        => $c[2],
                                            'default'     => $cdef,
                                            'distdefault' => $c[3],
                                            'desc'        => (defined(trim($match[3])) ? constant(trim($match[3])) : $match[3]) // find translation if constant used
                                          );
                    break;
            }
        }
    }

    if ($raw) {
        return $configRaw;
    } else {
        return $config;
    }
}


/**
 * Search the default value of an input
 *
 */
function setup_queryDefault($optname, $default) {
    switch ($optname) {
        case 'wkPath':
            // Path 1
            $test_path1 = $_SERVER['DOCUMENT_ROOT'] . rtrim(dirname($_SERVER['PHP_SELF']), '/') . '/';
            $test_path1_elems = explode('/',$test_path1);
            array_pop($test_path1_elems); // parent to setup dir (wiki system dir)
            array_pop($test_path1_elems); // parent to wiki system dir
            $test_path1 = implode(OS_FILE_SEPARATOR,$test_path1_elems).OS_FILE_SEPARATOR;
            
            if (@file_exists($test_path1 . 'index.php')) {
                return $test_path1;
            } else { // maybe web server alias used
                // Path 2
                $test_path2 = dirname(__FILE__);
                $test_path2_elems = explode(OS_FILE_SEPARATOR,$test_path2);
                array_pop($test_path2_elems); // parent to setup dir (wiki system dir)
                array_pop($test_path2_elems); // parent to wiki system dir
                $test_path2 = implode(OS_FILE_SEPARATOR,$test_path2_elems).OS_FILE_SEPARATOR;
                return $test_path2;
            }
            break;
        case 'wkHTTPPath':
            $pathElements = explode('/',rtrim(dirname($_SERVER['PHP_SELF']),'/'));
            array_pop($pathElements); // parent to setup dir (wiki system dir)
            array_pop($pathElements); // parent to wiki system dir
            $path = implode('/',$pathElements);
            return 'http://'.$_SERVER['HTTP_HOST'].$path.'/';
            break;
        case 'lang' :
            global $WK; // to retrieve old config value, if set
            if(!empty($WK)) 
                return $WK['lang'];
            else
                return 'fr';
            break;
        case 'renameFolders' :
        case 'renameFiles' :
        case 'moveFolders' :
        case 'moveFiles' :
        case 'deleteFolders' :
        case 'deleteFiles' :
            global $WK;
            if(!empty($WK) && isset($WK['defaultRights']))
                return (isset($WK['defaultRights'][$optname])) ? $WK['defaultRights'][$optname] : $default;
            else
                return $default;
            break;
        default:
            return $default;
            break;
    }
}


/**
 * Returns HTML Code for an input
 *
 */
function setup_guessInput($type, $name, $value='', $default='') {
    $input_content = '';
    switch ($type) {
        case 'bool' :
       
            $value = ($value == 'true') ? true : (($value=='false') ? false : $value);
            $input_content.= '<input id="radio_cfg_' . $name . '_yes" type="radio" name="' . $name . '" value="true" ';
            $input_content.= (($value == true) ? 'checked="checked"' : ''). ' /><label for="radio_cfg_' . $name . '_yes"> ' . YES . '</label>&nbsp;';
            $input_content.= '<input id="radio_cfg_' . $name . '_no" type="radio" name="' . $name . '" value="false" ';
            $input_content.= (($value == true) ? '' : 'checked="checked"'). ' /><label for="radio_cfg_' . $name . '_no"> ' . NO . '</label>';
    
             break;

        case 'protected' :
            $input_content.= '<input type="password" size="30" name="' . $name . '" value="' . htmlspecialchars($value) . '" />';
            break;
        case 'list' :
            preg_match_all("/([^\=]+)\=\>([^\,]+)\,?/i", $default, $res);
            $input_content.= '<select name="'. $name .'">';
            for ($x=0; $x<sizeof($res[1]); $x++) {
                $input_content.= sprintf('<option value="%s"%s>%s</option>'. "\n",
                                         $res[1][$x],
                                         (($res[1][$x] == $value) ? ' selected="selected"' : ''),
                                         $res[2][$x]);
            }
            $input_content.= '</select>';
            break;
        default :
            if(@is_array($value)) // for list of values, separated by ;
                $value = implode(';',$value);
            $input_content.= '<input type="text" size="30" name="' . $name . '" value="' . htmlspecialchars($value) . '" />';
            break;
    }
    return $input_content;
}

/**
 * Returns HTML Code for the config page
 *
 */
function setup_sprintConfigTemplate($t, $from = false, $_abort = false) {

ob_start();
?>
<script type="text/javascript" language="JavaScript">
function showConfig(id) {
    if (document.getElementById) {
        el = document.getElementById(id);
        if (el.style.display == 'none') {
            document.getElementById('option' + id).src = 'images/minus.png';
            el.style.display = '';
        } else {
            document.getElementById('option' + id).src = 'images/plus.png';
            el.style.display = 'none';
        }
    }
}

var state='none';
function showConfigAll(count) {
    if (document.getElementById) {
        for (i = 1; i <= count; i++) {
            document.getElementById('el' + i).style.display = state;
            document.getElementById('optionel' + i).src = '' + (state == '' ? 'images/minus.png' : 'images/plus.png');
        }

        if (state == '') {
            document.getElementById('optionall').src = 'images/minus.png';
            state = 'none';
        } else {
            document.getElementById('optionall').src = 'images/plus.png';
            state = '';
        }
    }
}

</script>

<form action="?" method="POST">
    <div>
        <!--<input type="hidden" name="wikiwig[adminModule]" value="installer" />-->
        <input type="hidden" name="installAction" value="check" />
        <br />
        
        <div align="right">
            <a style="border:0; text-decoration: none" href="#" onclick="showConfigAll(<?php echo count($t['categories']); ?>)" title="<?php echo TOGGLE_ALL; ?>"><img src="images/minus.png" id="optionall" alt="+/-" border="0" />&nbsp;<?php echo TOGGLE_ALL; ?></a></a><br />
        </div>
        <input type="submit" class="button" value="<?php echo CHECK_N_SAVE; ?>" />
<?php
        $el_count = 0;
        foreach ($t['categories'] as $key => $value) {
            $el_count++;
?>
        <table width="100%" cellspacing="2">
            <tr>
                <th align="left" colspan="2" style="padding-left: 15px;">
                    <a style="border:0; text-decoration: none" href="#" onclick="showConfig('el<?php echo $el_count; ?>'); return false" title="<?php echo TOGGLE_OPTION; ?>">
                        <img src="images/minus.png" id="optionel<?php echo $el_count; ?>" alt="+/-" border="0" />
                        &nbsp;<?php echo (defined($key) ? constant($key) : $key); ?>
                    </a>
                </th>
            </tr>

            <tr>
                <td>
                    <table style="display:;" width="100%" cellspacing="0" cellpadding="3" style="" id="el<?php echo $el_count; ?>">
                        <tr>
                            <td style="padding-left: 20px;" colspan="2">
                                <?php echo $t['descriptions'][$key]; ?>
                            </td>
                        </tr>


<?php
            for ($x=0; $x<count($value); $x++) {
                
                // if there is a valuelist, then use the value from there if exists
                if (@is_array($from) && isset($from[$value[$x]['name']])) {
                    $value[$x]['value'] = $from[$value[$x]['name']];
                }
                else // use the default value
                    $value[$x]['value'] = $value[$x]['default'];
?>
                        <tr>
                            <td style="border-bottom: 1px #000000 solid" align="left" valign="top" width="75%">
                                <strong><?php echo $value[$x]['longname']; ?></strong>
                                <br />
                                <span style="color: #5E7A94; font-size: 8pt;"><?php echo /*'(' . $value[$x]['type'] . ') ' .*/ $value[$x]['desc']; ?></span>
                            </td>
                            <td style="border-bottom: 1px #000000 solid; font-size: 8pt" align="left" valign="middle" width="25%">
                                <span style="white-space: nowrap"><?php echo setup_guessInput($value[$x]['type'], $value[$x]['name'], $value[$x]['value'], $value[$x]['distdefault']); ?></span>
                            </td>
                        </tr>
<?php
            }
?>
                    </table><br /><br />
                </td>
            </tr>
        </table>
<?php
        }
?>
        <input type="submit" class="button" value="<?php echo CHECK_N_SAVE; ?>" />
    </div>
</form>
<?php
    $content = ob_get_clean();
    return $content;
}


function prepare_fs_dir_path($var) {
    //$var = str_replace("/",OS_FILE_SEPARATOR,$var);
    //$var = trim($var,OS_FILE_SEPARATOR).OS_FILE_SEPARATOR;
    $var = str_replace(OS_FILE_SEPARATOR,"/",$var);
    $var = trim($var,"/")."/";
    $var = str_replace("//","/",$var);
    return $var;
}
/**
 * Parsed Paths : adds trailing slash or antislash
 * and format filesystem path with the separator of the filesystem
 *
 */
function setup_cleanConfigVars() {
    require_once '../compat.php';
    // Filesystem Dirs path
    if(isset($_POST['imagemanager_image_transform_lib_path'])) {
        $_POST['imagemanager_image_transform_lib_path'] = str_replace('"','',$_POST['imagemanager_image_transform_lib_path']);
        $_POST['imagemanager_image_transform_lib_path'] = prepare_fs_dir_path($_POST['imagemanager_image_transform_lib_path']);
    }
    
    // dir names
    $_POST['uploadDir'] = trim($_POST['uploadDir'],"/\\");
    $_POST['trashDir']  = trim($_POST['trashDir'],"/\\");
    $_POST['backupDir'] = trim($_POST['backupDir'],"/\\");
    
    // files path
    if(isset($_POST['imagemanager_default_thumbnail'])){
        //$_POST['imagemanager_default_thumbnail'] = str_replace("/",OS_FILE_SEPARATOR,$_POST['imagemanager_default_thumbnail']);
        $_POST['imagemanager_default_thumbnail'] = str_replace(OS_FILE_SEPARATOR,"/",$_POST['imagemanager_default_thumbnail']);
    }
    /*
    if(isset($_POST['sessionPath'])) {
        $_POST['sessionPath'] = prepare_fs_dir_path($_POST['sessionPath']);
    }
    */
}
/**
 * Checks parameters passed to the configuration setup page
 *
 */
function setup_checkConfiguration() {
		
    global $umask;
    $errs = array();
    setup_cleanConfigVars();
  // Check dirs
  // Upload Dir
    if (!is_dir(WK_PATH . $_POST['uploadDir'] ) && @mkdir(WK_PATH . $_POST['uploadDir'], $umask) !== true) {
        $errs[] = sprintf(WK_CONF_ERR_DIR_CREATE, $_POST['uploadDir']);
    }
    elseif (!is_writable(WK_PATH . $_POST['uploadDir'])) {
        $errs[] = sprintf(WK_CONF_ERR_DIR_NO_WRITE, $_POST['uploadDir']);
    }
  // Backup Dir
    if (!is_dir(WK_PATH . $_POST['backupDir'] ) && @mkdir(WK_PATH . $_POST['backupDir'], $umask) !== true) {
        $errs[] = sprintf(WK_CONF_ERR_DIR_CREATE, $_POST['backupDir']);
    }
    elseif (!is_writable(WK_PATH . $_POST['backupDir'])) {
        $errs[] = sprintf(WK_CONF_ERR_DIR_NO_WRITE, $_POST['backupDir']);
    }
  // Trash Dir
    if (!is_dir(WK_PATH . $_POST['trashDir'] ) && @mkdir(WK_PATH . $_POST['trashDir'], $umask) !== true) {
        $errs[] = sprintf(WK_CONF_ERR_DIR_CREATE, $_POST['trashDir']);
    }
    elseif (!is_writable(WK_PATH . $_POST['trashDir'])) {
        $errs[] = sprintf(WK_CONF_ERR_DIR_NO_WRITE, $_POST['trashDir']);
    }
    
    // Check DB
    @include_once '../lib/Wiki_DB.php';
    if(class_exists('Wiki_DB')) { // verify that the included file exists
        $db = Wiki_DB::getDB($_POST['dbType']);
        if($db->probe() === false) {
            foreach($db->probe_errors as $e)
                $errs[] = $e;
        }
    }
    else {
        $errs[] = WK_CONF_ERR_DB_LIB_NOT_FOUND;
    }
    // Check given prefix
    if(!preg_match('/[a-z_]+/si',$_POST['dbPrefix'])) {
        $errs[] = WK_CONF_ERR_DB_BAD_TBL_PREFIX;
    }
    
    //check if mail exist
 	if(!$_POST['adminMail']){
	     $errs[] = WK_CONF_ERR_MAIL;
   } 
    else{	    
	    $prefix=$_POST['dbPrefix'];
			$db  = Wiki_DB::getDB();
	    $number = rand(0,10000);
			// insertion
			$test_res_pages = $db->query("SELECT * FROM ".$_POST['dbPrefix']."pages;");
    	$test_res_users = $db->query("SELECT * FROM ".$_POST['dbPrefix']."utilisateurs;");
			
			if($test_res_pages==''){
					setup_installDatabase();
				}
					        /*Recorder in the base the admin nov 2005*/
    $db=Wiki_DB::getDB($_POST['dbType'],$_POST);
    $mdp =md5($_POST['adminPass']);
    $requeteVerif=" SELECT * FROM `{$_POST['dbPrefix']}utilisateurs` WHERE utilisateurs_nom LIKE 'admin_{$_POST['adminLogin']}'";
		$pageVerif=$db->query($requeteVerif);
		
		
		/*if(count($pageVerif) > 0){
			$errs[]=WK_LOGIN_ADMIN_BIS;
		}		
		else{*/
 
			$requete="INSERT INTO `{$prefix}utilisateurs` (utilisateurs_nom, utilisateurs_mdp, utilisateurs_couleur) VALUES ('admin_{$_POST['adminLogin']}', '".md5($_POST['adminPass'])."', '{$_POST['adminMail']}')";
			$res = $db->execute($requete);
				
				/*//confirm
				$titre = WK_PROFILE_MAIL_TITLE;
				$lien ="../form/wk_changer_profil.php?number=admin_{$number}&pseudo=" .$_POST['adminLogin'];
    		$corps = WK_PROFILE_MAIL_BODY_PART0 ."\n\n" . WK_PROFILE_MAIL_BODY_PART1 . " \"" . $WK['wkName'] . "\" " . WK_PROFILE_MAIL_BODY_PART2 . "<br/><a href='{$lien}'>" . WK_LABEL_CLICK_HERE. "</a>";//http://" . $_SERVER["SERVER_NAME"] . $_SERVER["PHP_SELF"] . "?number=$number&pseudo=" .$_POST['adminLogin'];

    		$result = mail($mail, $titre, $corps,
				"From: webmaster@wikiwig.sourceforge.net\n"
				."Reply-To: webmaster@{$_SERVER['SERVER_NAME']}\r\n"
				."X-Mailer: PHP/" . phpversion());

				if ($result){
					 $mail= WK_PROFILE_ERROR_SUCCESS_PART1 . " (" . $mail . ") " . WK_PROFILE_ERROR_SUCCESS_PART2;
				 }
				else {
					 $mail =WK_PROFILE_ERROR_MAILNOTSENT ." <br><br>" . $titre . "<br><br>" . $corps;
				}
				
			}*/
			if (isset($mail)){
				$mail= "<span id='rouge'>{$mail}</span>";
				@define('WK_Mail',"$mail");
	  	
		  }
		}
			


   	
    
    
    // Check Image Library
    if ( ($_POST['imagemanager_image_class'] != 'GD') && (!is_dir($_POST['imagemanager_image_transform_lib_path']) ) ) {
        $lib_name = ($_POST['imagemanager_image_class'] == 'IM') ? 'ImageMagick' : $_POST['imagemanager_image_class']; 
        $errs[] = sprintf(WK_CONF_ERR_GRAPH_LIB_NOT_FOUND, $lib_name, $_POST['imagemanager_image_transform_lib_path']);
    }
    

 		

    return (count($errs) > 0 ? $errs : '');
    
    
    
}

/**
 * Setups the database
 */
function setup_installDatabase() {
    $errs = array();
    @include_once '../lib/Wiki_DB.php';
    if(!class_exists('Wiki_DB')) { // verify that the included file exists
        $err[] = WK_CONF_ERR_DB_LIB_NOT_FOUND;
    }
    $db = Wiki_DB::getDB($_POST['dbType'],$_POST);
    $test_res_pages = $db->query("SELECT * FROM ".$_POST['dbPrefix']."pages;");
    $test_res_users = $db->query("SELECT * FROM ".$_POST['dbPrefix']."utilisateurs;");

  
    // Note on changing name for existent tables 
    // the query ALTER TABLE `wikiwig_pages` RENAME `NEW_PREFIX_pages` 
    // allow to change already installed
    // This behaviour is not still implemented , maybe in future versions
    $test_res = (($test_res_pages!==false) && ($test_res_users!==false));
    if($test_res !== false) {
        define('DB_ALREADY_INSTALLED',true);
        return true; // no install to do
    }
    define('DB_ALREADY_INSTALLED',false);
    // Tables Creation
    //$in_table = 0;
    //$queries = array();
    $sql = '';
    $fp = fopen('sql/wikiwig_tables.sql', 'r', 1);
    if (is_resource($fp)) {
        while (!@feof($fp)) {
            $line = trim(fgets($fp, 4096));
            if(substr($line,0,1)!='#')
                $sql.= $line;
            /*
            // Regexp Version
            if ($in_table) {
                $def .= $line;
                if (preg_match('/^\)\s*(type\=\S+)?\s*\;$/i', $line)) {
                    $in_table = 0;
                    array_push($queries, $def);
                }
            } else {
                if (preg_match('#^create table [`]?(wk_pages|wk_utilisateurs)[`]?\S+\s*\(#i', $line)) {
                    //echo 'In Table '.$line.'<br/>';
                    $in_table = 1;
                    $def = $line;
                }

                if (preg_match('#^create\s*(\{fulltext\}|unique)\s*index#i', $line)) {
                    array_push($queries, $line);
                }
            }
            */
        }
        fclose($fp);
    }
    $sql = str_replace('wk_pages',$_POST['dbPrefix'].'pages',$sql);
    $sql = str_replace('wk_utilisateurs',$_POST['dbPrefix'].'utilisateurs',$sql);
    $queries = preg_split('/(?<=;)(?=CREATE TABLE)/',$sql);
    $creation_res = true;
    foreach($queries as $query) {
        $creation_res = ($creation_res && $db->execute($query));
    }
    
    return $creation_res;
}

/**
 * Create the configuration File
 *
 */
function setup_updateConfiguration() {
    //if already installed, try backup old config file
    if(WK_INSTALLED === true){
        $old_file = dirname(__FILE__).'/../wk_config.php';
        @copy($old_file,$old_file.'.bak');
    }
    $path = dirname(__FILE__).'/../';
    $file = 'wk_config.php';
    $fp   = @fopen($path . $file, 'w+');
    
    if (!is_resource($fp)) {
        $errs[] = sprintf(WK_CONF_ERR_WRITE_CONF_FILE, $file);
        return $errs;
    }
    // special options arrays
    $filemanager = array();
    $imagemanager = array();
    
    /////
    // Safe_mode detection 
    // Based on not-allowed functions
    $output = @shell_exec('dir');
    $detected_safe_mode = (empty($output)) ? 'true' : 'false';
    
    // starting content of the file
    $config_content = "<?php\n";
    $config_content.= "/*\n";
    $config_content.= "\tWikiwig Configuration File\n";
    $config_content.= "\twritten on ".date('r')."\n";
    $config_content.= "*/\n";
    // use this var to replace the real value in the following config definitions
    // value of constants are not replaced in HereDoc strings
    $WK_SYSTEM_DIR = WK_SYSTEM_DIR;
    $WK_VERSION = WK_VERSION;
    $WK_HTTPPATH = WK_HTTPPATH;
    // In following HereDoc Strings
    // dont forget to use \\ to define a \
    // to use \$ when a $ should be written else vars will be replaced
    $config_content.= <<<INIT
// Set the version
\$WK['wikiwig_version'] = '$WK_VERSION';
\$WK['wikiwig_project_url'] = 'http://sourceforge.net/projects/wikiwig/';

// Safe_mode detected (change this if detection failed to find your current php config) 
\$WK['safe_mode'] = $detected_safe_mode;

\$WK['systemDir'] = '$WK_SYSTEM_DIR';
// Dynamic Detection of the system path of wikiwig
\$current_path = str_replace("\\\\","/",__FILE__);
\$WK['wkPath'] = substr(\$current_path,0,strpos(\$current_path,\$WK['systemDir'].'/'));

// HTTP path of wikiwig
\$WK['wkHTTPPath'] = '$WK_HTTPPATH';
\$WK['templatesDir'] = \$WK['systemDir'].'/template';
\$WK['templatesSystemDir'] = \$WK['templatesDir'].'/system';

// Configuration Options

INIT;
    if($_POST['sessionPath'] != '') {
        $config_content.= "\$WK['sessionPath'] = ".setup_formatConfigVar($_POST['sessionPath']).";\n";
        $config_content.= "session_save_path(\$WK['sessionPath']);\n";
    }
    $config_content.= "";
    // Retrieving code for the variables posted to the config page
    $rights_vars = array('renameFolders','renameFiles','moveFolders','moveFiles','deleteFolders','deleteFiles');
    foreach($_POST as $key => $val){
        if($key!='installAction' && $key!='hiddenDirs' && !in_array($key,$rights_vars))
            $config_content.= "\$WK['".$key."'] = ".setup_formatConfigVar($val).";\n";
        // special cases
        elseif($key == 'hiddenDirs'){
            $val = str_replace(";","','",$val); // make an array list
            $config_content.= "\$WK['".$key."'] = array('".$val."');\n";
        }
        elseif(in_array($key,$rights_vars)) {
            $config_content.= "\$WK['defaultRights']['".$key."'] = ".setup_formatConfigVar($val).";\n";
        }
        
        //pas Enregistrer le nom Admin & login & mail dans config mais dans base
        if  (($key =='adminLogin')||($key =='adminPass') || ($key =='css_wiki')){
	         $config_content.="//enregistrement dans la base ";
       }
        // Add some config vars depending on the key
        if($key == 'dbPrefix'){
            $config_content.= "\$WK['table_pages'] = \$WK['dbPrefix'].'pages';\n";
            $config_content.= "\$WK['table_utilisateurs'] = \$WK['dbPrefix'].'utilisateurs';\n";
        }
        if(strstr($key,'filemanager_')){
            $filemanager[substr($key,strlen('filemanager_'))] = $val;
        }
        if(strstr($key,'imagemanager_')){
            $imagemanager[substr($key,strlen('imagemanager_'))] = $val;
        }
    }
    
    // Backward compatibility
    // and use of plugins constants
    $config_content.= "\n//File Manager Plugin\n//Redundant variables to respect FM Plugin Configuration\n";
    $config_content.= "\$MY_DOCUMENT_ROOT = \$WK['wkPath'].\$WK['uploadDir'];\n";
    $config_content.= "\$MY_BASE_URL = \$WK['wkHTTPPath'].\$WK['uploadDir'];\n";
    $config_content.= "\$MY_URL_TO_OPEN_FILE = \$WK['wkHTTPPath'].\$WK['uploadDir'];\n";
    $config_content.= "\$MY_DENY_EXTENSIONS = array('php', 'php3', 'php4', 'phtml', 'shtml', 'cgi', 'pl');\n";
    foreach($filemanager as $k => $v){
        $k_up = strtoupper($k);
        if($k_up == 'ALLOW_EXTENSIONS'){
            $v = str_replace(";","','",$v); // make an array list
            $config_content.= "\$MY_".$k_up." = array('".$v."');\n";
            $config_content.= "\$MY_LIST_EXTENSIONS = array('".$v."');\n";
        }
        else
            $config_content.= "\$MY_".$k_up." = ".setup_formatConfigVar($v).";\n";
    }
    $config_content.= "\n//Image Manager Plugin\n//Redundant variables to respect IM Plugin Configuration\n";
    $config_content.= "\$IMConfig['base_dir'] = \$WK['wkPath'].\$WK['uploadDir'];\n";
    $config_content.= "\$IMConfig['base_url'] = \$WK['wkHTTPPath'].\$WK['uploadDir'];\n";
    $config_content.= "\$IMConfig['safe_mode'] = \$WK['safe_mode'];\n";
    foreach($imagemanager as $k => $v){
        if($k == 'image_class'){
            $config_content.= "define('IMAGE_CLASS', ".setup_formatConfigVar($v).");\n";
        }
        elseif($k == 'image_transform_lib_path'){
            $config_content.= "define('IMAGE_TRANSFORM_LIB_PATH', ".setup_formatConfigVar($v).");\n";
        }
        else
            $config_content.= "\$IMConfig['".$k."'] = ".setup_formatConfigVar($v).";\n";
    }
    
    $config_content.=<<<INCLUSIONS
// PHP Versions compatibility script
@include_once(dirname(__FILE__).'/compat.php');

// Language definitions
@include_once(dirname(__FILE__).'/wk_lang.php');

INCLUSIONS;
    $config_content.= "?>\n";
    fwrite($fp, $config_content);
    fclose($fp);
    
    umask(0000);
    @chmod($path . $file, 0777);
    return true;
}

/**
 * Format values of variables for configuration file
 *
 */
function setup_formatConfigVar($val){
    // for int or boolean, just return the value
    if(($val == 'false') || ($val == 'true') || (is_numeric($val))) {
        return $val;
    }
    else{ // string => escape " and \ + add ""
        //return "'".addslashes($val)."'";
        $val = str_replace('"','\"',$val);
        $val = str_replace("\\","\\\\",$val);
        return '"'.$val.'"';
    }
}


				

?>
